Attribute VB_Name = "modTextBoxes"
'Free to use and distribute, if you leave the following intact
'"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
'VBClassic is better than VB.NET for desktop apps
'  Some simple techniques such as -
'  - Data encapsulation (tight scope)
'  - etc,
'can avoid the problems of the past, and achieve Code Re-use
'without adding massive 'OO' complexity to your apps.
'
'Someone will develop a simpler approach for web development, in the future.
'
'Rob Crombie ( rob@crombie.com )



'The two Public procedures are Functions, but in this example they are treated as Subs.
' You may have some future use, where the calling Form wishes to be advised of errors.
Option Explicit

'You don't have to use the return Boolean value (This procedure will cancel the keystroke for you)
Public Function NumOnlyKeyPress(tBox As TextBox, ByRef KAsc As Integer) As Boolean
 
  'Allow   +   -    ,   .                 0 to 9
  If (KAsc >= 43 And KAsc <= 46) Or (KAsc >= 48 And KAsc <= 57) Or (KAsc = 8) Then
    NumOnlyKeyPress = True
    Exit Function   '<=== Exit
  End If
    
  'Decide whether to set error flag.  Just discard  13(Enter)  22(Ctrl+V)
  Select Case KAsc
    Case 13, 22
     'Dscard without error
      NumOnlyKeyPress = True
    Case Else
      NumOnlyKeyPress = False
  End Select
  'Cancel the keystroke
  KAsc = 0
  
End Function


'This repeats some checking done by KeyPress, as user may have used right click Paste
'You don't have to use the return Boolean value (This procedure will cancel the latest changes for you)
Public Function NumOnlyChange(tBox As TextBox, ByRef sWas As String) As Boolean
 Const sValChars As String = "1234567890-+,."
 Static sThou As String  'Now using auto detector
 Static sDec As String   'Now using auto detector
 Dim sWork As String
 Dim OK As Boolean
 Dim i As Integer
  
  'Auto detect the Thou and Dec characters (only runs first time this proc is called)
  If sDec = "" Then
    DetermineDecimalCharacter sThou, sDec
  End If
 
  tBox.Text = Trim(tBox.Text)
  sWork = tBox.Text
  'Ensure all characters are allowed characters
  For i = 1 To Len(sWork)
    If InStr(sValChars, Mid(sWork, i, 1)) = 0 Then
      ReverseUsersTypingAndPositionCaret tBox, sWas
      NumOnlyChange = False
      Exit Function   '<=== Exit
    End If
  Next i
  
  'NB sWork is stripped of + - Decimal(and fraction)  once they are found.
  
  'Call Function to ensure signs absent or correct (if correct, then remove it
  OK = CheckAndRemovePlusOrMinus(sWork)
  If OK = False Then
    MsgBox "Problem with the Signs", vbQuestion, "   +   or   -   HAS PROBLEMS"
    ReverseUsersTypingAndPositionCaret tBox, sWas
    NumOnlyChange = False
    Exit Function   '<=== Exit
  End If
  'Call Function to ensure decimal character correct (then remove it, and fraction)
  OK = CheckAndRemoveDecimals(sWork, sThou, sDec)
  If OK = False Then
    MsgBox "Problem with the Decimal", vbQuestion, "   DECIMAL   HAS PROBLEMS"
    ReverseUsersTypingAndPositionCaret tBox, sWas
    NumOnlyChange = False
    Exit Function   '<=== Exit
  End If
  'Call function to ensure thousand characters are in correct positions
  OK = CheckThousands(sWork, sThou)
  If OK = False Then
    'Warning only
    tBox.BackColor = RGB(255, 180, 180) 'No choice but to allow user to keep typing (with red warning)
  Else
    tBox.BackColor = vbWhite
  End If
  'Since I did not roll back the User's change, then remember latest contents
  sWas = tBox.Text
  NumOnlyChange = True
End Function

'This only allows one + or - on left of sIN
'NB sIN has the + or - removed
Private Function CheckAndRemovePlusOrMinus(ByRef sIN) As Boolean
  'If only 1 char long then cannot be an error
  If Len(sIN) <= 1 Then
    CheckAndRemovePlusOrMinus = True
    Exit Function   '<=== Exit
  End If
  If Left(sIN, 1) = "+" Or Left(sIN, 1) = "-" Then
    'Remove sign
    sIN = Mid(sIN, 2) 'NB this now passed back (without the leading sign)
                      '   (but an error can still be flagged)
    'If another sign still present then Error.
    If InStr(sIN, "+") > 0 Or InStr(sIN, "-") > 0 Then
      CheckAndRemovePlusOrMinus = False
      Exit Function   '<=== Exit
    End If
  Else
    'May be a sign, but not on left
    If InStr(sIN, "+") > 0 Or InStr(sIN, "-") > 0 Then
      CheckAndRemovePlusOrMinus = False
      Exit Function   '<=== Exit
    End If
  End If
  'If gets to here, then  no sign,  or    there was a sign on left NOW GONE.
  CheckAndRemovePlusOrMinus = True
End Function

'Pass me the decimal character that is used in this country ( usually . )
'NB sIN has the decimal and following digits, removed.
Private Function CheckAndRemoveDecimals(ByRef sIN, sThou As String, sDec As String) As Boolean
 Dim pos As Integer
 Dim sWhole As String    '(store the Left of decimal)
 Dim sFraction As String '(store the right of the decimal)
  'If only 1 char long then cannot be an error
  If Len(sIN) <= 1 Then
    CheckAndRemoveDecimals = True
    Exit Function   '<=== Exit
  End If
  pos = InStr(sIN, sDec)
  If pos > 0 Then
    sWhole = Left(sIN, pos - 1)
    sFraction = Mid(sIN, pos + 1)
  Else
    sWhole = sIN
  End If
  'If another decimal present then we have a problem
  If InStr(sWhole, sDec) > 0 Or InStr(sFraction, sDec) > 0 Then
    CheckAndRemoveDecimals = False
    Exit Function   '<=== Exit
  End If
  'If Thou after the decimal, we have a problem
  If InStr(sFraction, sThou) > 0 Then
    CheckAndRemoveDecimals = False
    Exit Function   '<=== Exit
  End If
  
  sIN = sWhole  'This is passed back without the decimal and following chars
  CheckAndRemoveDecimals = True
End Function


'Pass me the thousand character that is used in this country ( usually , )
'NB Signs and decimals are already stripped away prior to calling me
'Any Comma error does not trigger data reversal (A Warning instead)
' Hard to trigger error as user may be in the process of typing.
' So trigger Warning only, NO ERRORS
'Use thou char to split into an array.
'If first element is  , then Warn
'If last element is not 3 long then Warn
'If middle elements are not 3 long then Warn
Private Function CheckThousands(ByRef sIN, sThou As String) As Boolean
 Dim i As Integer
 Dim sArray() As String
  'If only 1 char long then Exit, but still Warn if it is the thou character
  If Len(sIN) <= 1 Then
    If sIN = sThou Then
      CheckThousands = False 'Warn that only a thou character present
    Else
      CheckThousands = True
    End If
    Exit Function   '<=== Exit
  End If
  'If no Thou characters present then exit
  If InStr(sIN, sThou) = 0 Then
    CheckThousands = True
    Exit Function   '<=== Exit
  End If
  'Split into an array
  sArray = Split(sIN, sThou)
  'First element should be 1 or 2 or 3 long
  If Len(sArray(0)) = 0 Or Len(sArray(0)) > 3 Then
    CheckThousands = False  'Warn
    Exit Function   '<=== Exit
  End If
  'Last element should be 3 long
  If UBound(sArray) > 0 Then
    If Len(sArray(UBound(sArray))) <> 3 Then
      CheckThousands = False  'Warn
      Exit Function   '<=== Exit
    End If
  End If
  'If middle elements present (more than 2 elements), they should all be 3 long
  If UBound(sArray) > 1 Then
    For i = 1 To (UBound(sArray) - 1)
      If Len(sArray(i)) <> 3 Then
        CheckThousands = False  'Warn
        Exit Function   '<=== Exit
      End If
    Next i
  End If
  CheckThousands = True  'No Warning
End Function

Private Sub ReverseUsersTypingAndPositionCaret(tBox As TextBox, sWas As String)
  tBox.Text = sWas
  tBox.SelStart = Len(sWas)
End Sub



Private Sub DetermineDecimalCharacter(ByRef sThou As String, ByRef sDec As String)
 Dim sNumber As String
  sNumber = Format(1000, "#,##0.00")
  sThou = Mid(sNumber, 2, 1)
  sDec = Mid(sNumber, 6, 1)
End Sub
